@@ -19,8 +19,15 @@ package mit.alibaba.nuidemo;
 
 import android.app.Activity;
 import android.content.Intent;
+import android.graphics.Color;
+import android.graphics.Typeface;
 import android.os.Bundle;
+import android.text.Editable;
+import android.text.Spannable;
 import android.text.TextUtils;
+import android.text.style.BackgroundColorSpan;
+import android.text.style.ForegroundColorSpan;
+import android.text.style.StyleSpan;
 import android.view.View;
 import android.widget.AdapterView;
 import android.widget.ArrayAdapter;
@@ -28,11 +35,13 @@ import android.widget.Button;
 import android.widget.EditText;
 
 import android.util.Log;
+import android.widget.ProgressBar;
 import android.widget.Spinner;
 import android.widget.Switch;
 import android.widget.TextView;
 import android.widget.Toast;
 
+import com.alibaba.fastjson.JSON;
 import com.alibaba.fastjson.JSONArray;
 import com.alibaba.fastjson.JSONException;
 import com.alibaba.fastjson.JSONObject;
@@ -46,10 +55,11 @@ import java.io.OutputStream;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
+import java.util.Locale;
 import java.util.Map;
 
 // 本样例展示在线语音合成CosyVoice（流式输入流式输出）使用方法
-// Android SDK 详细说明：
+// Android SDK 详细说明：https://help.aliyun.com/zh/model-studio/cosyvoice-android-sdk
 // WebSocket API: https://help.aliyun.com/zh/model-studio/cosyvoice-websocket-api
 public class DashCosyVoiceStreamTtsActivity extends Activity implements View.OnClickListener, AdapterView.OnItemSelectedListener {
     private static final String TAG = "CosyVoiceStreamTts";
@@ -58,29 +68,32 @@ public class DashCosyVoiceStreamTtsActivity extends Activity implements View.OnC
     private String g_url = "";
 
     NativeNui stream_input_tts_instance = new NativeNui(Constants.ModeType.MODE_STREAM_INPUT_TTS);
-    final static String CN_TEXT = "唧唧复唧唧，木兰当户织。不闻机杼声，唯闻女叹息。\n" +
-            "问女何所思，问女何所忆。女亦无所思，女亦无所忆。\n" +
-            "昨夜见军帖，可汗大点兵，军书十二卷，卷卷有爷名。\n" +
-            "阿爷无大儿，木兰无长兄，愿为市鞍马，从此替爷征。\n" +
-            "东市买骏马，西市买鞍鞯，南市买辔头，北市买长鞭。旦辞爷娘去，暮宿黄河边，不闻爷娘唤女声，但闻黄河流水鸣溅溅。\n" +
-            "旦辞黄河去，暮至黑山头，不闻爷娘唤女声，但闻燕山胡骑鸣啾啾。万里赴戎机，关山度若飞。朔气传金柝，寒光照铁衣。将军百战死，壮士十年归。\n" +
-            "归来见天子，天子坐明堂。策勋十二转，赏赐百千强。可汗问所欲，木兰不用尚书郎，愿驰千里足，送儿还故乡。爷娘闻女来，出郭相扶将；阿姊闻妹来，当户理红妆；小弟闻姊来，磨刀霍霍向猪羊。\n" +
-            "开我东阁门，坐我西阁床。脱我战时袍，著我旧时裳。当窗理云鬓，对镜帖花黄。出门看火伴，火伴皆惊忙：同行十二年，不知木兰是女郎。\n" +
-            "雄兔脚扑朔，雌兔眼迷离；双兔傍地走，安能辨我是雄雌？";
+    final static String CN_TEXT = "唧唧复唧唧，木兰当户织。\n不闻机杼声，唯闻女叹息。\n" +
+            "问女何所思，问女何所忆。\n女亦无所思，女亦无所忆。\n昨夜见军帖，可汗大点兵，军书十二卷，卷卷有爷名。\n" +
+            "阿爷无大儿，木兰无长兄，愿为市鞍马，从此替爷征。\n东市买骏马，西市买鞍鞯，南市买辔头，北市买长鞭。\n" +
+            "旦辞爷娘去，暮宿黄河边，不闻爷娘唤女声，但闻黄河流水鸣溅溅。\n" +
+            "旦辞黄河去，暮至黑山头，不闻爷娘唤女声，但闻燕山胡骑鸣啾啾。\n" +
+            "万里赴戎机，关山度若飞。\n朔气传金柝，寒光照铁衣。\n将军百战死，壮士十年归。\n" +
+            "归来见天子，天子坐明堂。\n策勋十二转，赏赐百千强。\n可汗问所欲，木兰不用尚书郎，愿驰千里足，送儿还故乡。\n" +
+            "爷娘闻女来，出郭相扶将；\n阿姊闻妹来，当户理红妆；\n小弟闻姊来，磨刀霍霍向猪羊。\n" +
+            "开我东阁门，坐我西阁床。\n脱我战时袍，著我旧时裳。\n当窗理云鬓，对镜帖花黄。\n" +
+            "出门看火伴，火伴皆惊忙：\n同行十二年，不知木兰是女郎。\n雄兔脚扑朔，雌兔眼迷离；\n双兔傍地走，安能辨我是雄雌？";
     final static String SINGLE_CN_TEXT = "<speak>唧唧复唧唧，木兰当户织。<break time=\"500ms\"/>不闻机杼声，唯闻女叹息。</speak>";
 
     private final Map<String, List<String>> paramMap = new HashMap<>();
     private Button ttsStartBtn, ttsPlayBtn, ttsStopBtn, ttsCancelBtn, ttsClearTextBtn;
-    private Switch mSaveAudioSwitch;
+    private Switch mSaveAudioSwitch, mSubtitlesSwitch;
     private Spinner mFormatSpin, mModelSpin, mFontSpin;
     private Spinner mPitchSpin, mSpeedSpin, mVolumeSpin, mBitRateSpin;
     private Spinner mSeedSpin, mStyleSpin, mLanguageHintsSpin, mInstructionSpin;
     private EditText ttsEditView;
     private TextView eventView;
+    private TextView playedView;
+    private ProgressBar playedBar;
 
     private boolean is_started = false;
     private String debug_path;
-    private String mEncodeType = "longanrou"; // AudioPlayer中AudioTrack只能播放PCM格式，其他音频格式请另外编写播放器代码
+    private String mEncodeType = "pcm"; // AudioPlayer中AudioTrack只能播放PCM格式，其他音频格式请另外编写播放器代码
     private String curTaskId = "";
     private String mSynthesisAudioFilePath = "";
     private OutputStream mSynthesisAudioFile = null;
@@ -88,19 +101,23 @@ public class DashCosyVoiceStreamTtsActivity extends Activity implements View.OnC
     private long startTimestamp = 0;
     private long startedTimestamp = 0;
     private long firstDataTimestamp = 0;
+    private long writeDataBytes = 0;
     private boolean firstDataFlag = true;
 
+    private Utils.ShowSubtitleInfo show_text_info = new Utils.ShowSubtitleInfo();
+    private Utils.HighLightWordList high_light_show = new Utils.HighLightWordList();
+
     //  AudioPlayer默认采样率是16000, 需要重设置采样率
     private int SAMPLE_RATE = 24000;
     private final AudioPlayer mAudioTrack = new AudioPlayer(new AudioPlayerCallback() {
         @Override
         public void playStart() {
-            Log.i(TAG, "start play");
+            Log.i(TAG, "player start");
         }
 
         @Override
         public void playOver() {
-            Log.i(TAG, "play over");
+            Log.i(TAG, "player over");
             runOnUiThread(new Runnable() {
                 @Override
                 public void run() {
@@ -113,7 +130,57 @@ public class DashCosyVoiceStreamTtsActivity extends Activity implements View.OnC
         }
 
         @Override
-        public void playSoundLevel(int level) {
+        public void playSoundLevel(int level) {}
+
+        @Override
+        public void playInfo(String info) {
+            if (mSubtitlesSwitch.isChecked()) {
+//                Log.d(TAG, "player info:" + info);
+                boolean changed = high_light_show.flushHighLight(info);
+                if (changed) {
+                    JSONObject obj = JSON.parseObject(info);
+                    int played_time_ms = obj.getIntValue("end_time");
+                    int played_secs = played_time_ms / 1000;
+                    int played_ms = played_time_ms % 1000;
+                    int total_time_ms = obj.getIntValue("total_time");
+                    int total_secs = total_time_ms / 1000;
+                    int total_ms = total_time_ms % 1000;
+                    String progress_result = String.format(Locale.getDefault(),
+                            "%d.%03d/%d.%03d", played_secs, played_ms, total_secs, total_ms);
+
+                    runOnUiThread(new Runnable() {
+                        @Override
+                        public void run() {
+                            playedView.setText(progress_result);
+                            playedBar.setMax(total_time_ms);
+                            playedBar.setProgress(played_time_ms);
+
+                            ttsEditView.setText(show_text_info.show_text);
+                            Editable editable = ttsEditView.getText();
+                            applyHighlight(editable);
+                        }
+                    });
+                }
+            } else {
+                JSONObject obj = JSON.parseObject(info);
+                int played_time_ms = obj.getIntValue("end_time");
+                int played_secs = played_time_ms / 1000;
+                int played_ms = played_time_ms % 1000;
+                int total_time_ms = obj.getIntValue("total_time");
+                int total_secs = total_time_ms / 1000;
+                int total_ms = total_time_ms % 1000;
+                String progress_result = String.format(Locale.getDefault(),
+                        "%d.%03d/%d.%03d", played_secs, played_ms, total_secs, total_ms);
+
+                runOnUiThread(new Runnable() {
+                    @Override
+                    public void run() {
+                        playedView.setText(progress_result);
+                        playedBar.setMax(total_time_ms);
+                        playedBar.setProgress(played_time_ms);
+                    }
+                });
+            }
         }
     });
 
@@ -150,15 +217,16 @@ public class DashCosyVoiceStreamTtsActivity extends Activity implements View.OnC
 
     @Override
     protected void onDestroy() {
+        Log.i(TAG, "onDestroy");
         super.onDestroy();
         mAudioTrack.stop();
         mAudioTrack.releaseAudioTrack();
         if (is_started) {
-            stream_input_tts_instance.stopStreamInputTts();
+            Log.i(TAG, "cancel stream input tts ...");
+            stream_input_tts_instance.cancelStreamInputTts();
             is_started = false;
         }
         stream_input_tts_instance.release();
-        stream_input_tts_instance = null;
     }
 
     private void getModelList() {
@@ -167,7 +235,6 @@ public class DashCosyVoiceStreamTtsActivity extends Activity implements View.OnC
         model.add("cosyvoice-v3");
         model.add("cosyvoice-v3-plus");
         model.add("cosyvoice-v1");
-        model.add("cosyvoice-v1");
         ArrayAdapter<String> spinnerFormat = new ArrayAdapter<String>(
                 DashCosyVoiceStreamTtsActivity.this,
                 android.R.layout.simple_spinner_dropdown_item, model);
@@ -331,6 +398,7 @@ public class DashCosyVoiceStreamTtsActivity extends Activity implements View.OnC
     }
 
     private void initUIWidgets() {
+        playedBar = (ProgressBar) findViewById(R.id.progressBar2);
         ttsEditView = (EditText) findViewById(R.id.tts_content);
         ttsStartBtn = (Button) findViewById(R.id.tts_start_btn);
         ttsPlayBtn = (Button) findViewById(R.id.tts_play_btn);
@@ -344,11 +412,19 @@ public class DashCosyVoiceStreamTtsActivity extends Activity implements View.OnC
         ttsStopBtn.setEnabled(true);
         ttsCancelBtn.setEnabled(true);
 
+        playedView = (TextView) findViewById(R.id.textView23);
+        playedView.setEnabled(false);
+
         eventView = (TextView) findViewById(R.id.textView15);
         eventView.setEnabled(false);
 
         mSaveAudioSwitch = (Switch) findViewById(R.id.save_audio_switch6);
         mSaveAudioSwitch.setVisibility(View.VISIBLE);
+        mSaveAudioSwitch.setChecked(false);
+
+        mSubtitlesSwitch = (Switch) findViewById(R.id.save_subtitles);
+        mSubtitlesSwitch.setVisibility(View.VISIBLE);
+        mSubtitlesSwitch.setChecked(false);
 
         mPitchSpin = (Spinner) findViewById(R.id.tts_set_pitch_spin);
         mPitchSpin.setOnItemSelectedListener(this);
@@ -425,18 +501,24 @@ public class DashCosyVoiceStreamTtsActivity extends Activity implements View.OnC
                                         if (!sentence.trim().isEmpty()) {
                                             // 单次文本长度和多次累计文本长度均有限制，详细请查下以下文档。
                                             // 文本长度限制: https://help.aliyun.com/zh/model-studio/cosyvoice-websocket-api
-                                            stream_input_tts_instance.sendStreamInputTts(sentence);
+                                            if (stream_input_tts_instance == null) {
+                                                break;
+                                            } else {
+                                                stream_input_tts_instance.sendStreamInputTts(sentence);
+                                            }
                                             if (!is_started) {
                                                 break;
                                             }
                                             try {
-                                                Thread.sleep(500);
+                                                Thread.sleep(200);
                                             } catch (InterruptedException e) {
                                                 e.printStackTrace();
                                             }
                                         }
                                     }
-                                    stream_input_tts_instance.stopStreamInputTts();
+                                    if (stream_input_tts_instance != null) {
+                                        stream_input_tts_instance.stopStreamInputTts();
+                                    }
                                 }
                             }
                         }).start();
@@ -490,6 +572,27 @@ public class DashCosyVoiceStreamTtsActivity extends Activity implements View.OnC
         });
     }
 
+    private void applyHighlight(Editable s) {
+        if (s == null) return;
+        int index = 0;
+        int end = high_light_show.high_light_begin_index > s.length() ?
+                s.length() :
+                high_light_show.high_light_begin_index;
+        if (end >= 0 && end > index) {
+            // 添加背景色高亮
+            s.setSpan(new BackgroundColorSpan(Color.YELLOW),
+                    index, end,
+                    Spannable.SPAN_EXCLUSIVE_EXCLUSIVE);
+            // 添加文字颜色或粗体
+            s.setSpan(new ForegroundColorSpan(Color.RED),
+                    index, end,
+                    Spannable.SPAN_EXCLUSIVE_EXCLUSIVE);
+            s.setSpan(new StyleSpan(Typeface.BOLD),
+                    index, end,
+                    Spannable.SPAN_EXCLUSIVE_EXCLUSIVE);
+        }
+    }
+
     private String genTicket() {
         String str = "";
         try {
@@ -522,7 +625,7 @@ public class DashCosyVoiceStreamTtsActivity extends Activity implements View.OnC
             }
 
             //过滤SDK内部日志通过回调送回到用户层
-            object.put("log_track_level", String.valueOf(Constants.LogLevel.toInt(Constants.LogLevel.LOG_LEVEL_INFO)));
+            object.put("log_track_level", String.valueOf(Constants.LogLevel.toInt(Constants.LogLevel.LOG_LEVEL_NONE)));
 
             str = object.toString();
         } catch (JSONException e) {
@@ -532,18 +635,20 @@ public class DashCosyVoiceStreamTtsActivity extends Activity implements View.OnC
         return str;
     }
 
-    private String genParameters() {
+    private String genParameters(boolean long_play_flag) {
         String str = "";
         try {
             String font_name = mFontSpin.getSelectedItem().toString();
             String[] parts = font_name.split("-");
             if (parts.length > 1) {
-                mEncodeType = parts[0];
+                font_name = parts[0];
+            } else {
+                font_name = "longyumi_v2";
             }
 
             JSONObject object = new JSONObject();
             object.put("model", mModelSpin.getSelectedItem().toString());
-            object.put("voice", mEncodeType);
+            object.put("voice", font_name);
             /**
              * 音频编码格式。
              * 所有模型均支持的编码格式：pcm、wav和mp3（默认）
@@ -596,7 +701,35 @@ public class DashCosyVoiceStreamTtsActivity extends Activity implements View.OnC
                 }
             }
 
-            if (mModelSpin.getSelectedItem().toString().equals("cosyvoice-v3") ||
+            if (long_play_flag) {
+                /**
+                 * 是否开启SSML功能。
+                 * 该参数设为 true 后，仅允许发送一次文本，支持纯文本或包含SSML的文本。
+                 * */
+                object.put("enable_ssml", true);
+            }
+
+            /* 返回音频数据的封装中带上AIGC的标记, 默认关闭 */
+//            object.put("enable_aigc_tag", true);
+
+            // 设置文档中不存在的参数, key为custom_params, value以json string的形式设置参数
+            // 如下示例传入{parameters:{"custom_param_flag":true},"user111":"111"}表示在payload下添加参数
+            // payload.user111 : 111
+            // payload.parameters.custom_param_flag : true
+//            JSONObject custom_params = new JSONObject();
+//            JSONObject parameters = new JSONObject();
+//            custom_params.put("user111", "111");
+//            parameters.put("custom_param_flag", true);
+//            custom_params.put("parameters", parameters);
+//            object.put("custom_params", custom_params);
+
+            if (mModelSpin.getSelectedItem().toString().equals("cosyvoice-v2")) {
+                /**
+                 * 是否开启字级别时间戳，默认为false关闭。
+                 * 仅cosyvoice-v2支持该功能。
+                 * */
+                object.put("word_timestamp_enabled", mSubtitlesSwitch.isChecked());
+            } else if (mModelSpin.getSelectedItem().toString().equals("cosyvoice-v3") ||
                     mModelSpin.getSelectedItem().toString().equals("cosyvoice-v3-plus")) {
                 /**
                  * seed:
@@ -609,14 +742,14 @@ public class DashCosyVoiceStreamTtsActivity extends Activity implements View.OnC
                 }
 
                 /**
-                 * style:
+                 * style: 已经废弃
                  * 调整风格。默认值0。取值应为大于等于0的整数。
                  * 仅cosyvoice-v3、cosyvoice-v3-plus支持该功能。
                  * */
-                String style = mStyleSpin.getSelectedItem().toString();
-                if (!TextUtils.isEmpty(style) && !style.trim().isEmpty()) {
-                    object.put("style", Integer.parseInt(style));
-                }
+//                String style = mStyleSpin.getSelectedItem().toString();
+//                if (!TextUtils.isEmpty(style) && !style.trim().isEmpty()) {
+//                    object.put("style", Integer.parseInt(style));
+//                }
 
                 /**
                  * instruction:
@@ -645,7 +778,9 @@ public class DashCosyVoiceStreamTtsActivity extends Activity implements View.OnC
                 if (!TextUtils.isEmpty(languageHints) && !languageHints.trim().isEmpty()) {
                     JSONArray language_hints_array = new JSONArray();
                     language_hints_array.add(languageHints);
-                    object.put("language_hints", language_hints_array.toString());
+                    // language_hints 是String或者Array都可以
+                    object.put("language_hints", language_hints_array);
+//                    object.put("language_hints", language_hints_array.toString());
                 }
             }
 
@@ -667,7 +802,7 @@ public class DashCosyVoiceStreamTtsActivity extends Activity implements View.OnC
             public void onStreamInputTtsEventCallback(INativeStreamInputTtsCallback.StreamInputTtsEvent event,
                                                       String task_id, String session_id, int ret_code,
                                                       String error_msg, String timestamp, String all_response) {
-                Log.i(TAG, "stream input tts event(" + event + ") task id(" + task_id + ") retCode(" + ret_code + ") errMsg(" + error_msg + ")");
+                Log.d(TAG, "stream input tts event(" + event + ") task id(" + task_id + ") retCode(" + ret_code + ") errMsg(" + error_msg + ")");
                 if (event == StreamInputTtsEvent.STREAM_INPUT_TTS_EVENT_SYNTHESIS_STARTED) {
                     Log.i(TAG, "STREAM_INPUT_TTS_EVENT_SYNTHESIS_STARTED");
                     startedTimestamp = System.currentTimeMillis();
@@ -682,7 +817,11 @@ public class DashCosyVoiceStreamTtsActivity extends Activity implements View.OnC
                             }
                             if (mSynthesisAudioFile == null && !curTaskId.isEmpty()) {
                                 try {
-                                    mSynthesisAudioFilePath = debug_path + "/" + "tts_task_id_" + curTaskId + "." + mEncodeType;
+                                    String format = mFormatSpin.getSelectedItem().toString();
+                                    if (format.equals("opus") || format.equals("mp3")) {
+                                        format = "pcm";
+                                    }
+                                    mSynthesisAudioFilePath = debug_path + "/" + "tts_task_id_" + curTaskId + "." + format;
                                     Log.i(TAG, "save tts data into " + mSynthesisAudioFilePath);
                                     mSynthesisAudioFile = new FileOutputStream(mSynthesisAudioFilePath, true);
                                 } catch (IOException e) {
@@ -695,8 +834,42 @@ public class DashCosyVoiceStreamTtsActivity extends Activity implements View.OnC
                     }
                     mAudioTrack.play();
                     Log.i(TAG, "start play");
+                    show_text_info.clear();
+                    high_light_show.clear();
                 } else if (event == StreamInputTtsEvent.STREAM_INPUT_TTS_EVENT_SENTENCE_SYNTHESIS) {
-                    Log.i(TAG, "STREAM_INPUT_TTS_EVENT_SENTENCE_SYNTHESIS:" + all_response);
+                    /* 字幕信息可以从all_response中解析，也可从timestamp中解析 */
+                    if (mSubtitlesSwitch.isChecked()) {
+                        if (false) {
+                            JSONObject root = JSON.parseObject(all_response);
+                            if (root == null) {
+                                return;
+                            }
+                            JSONObject payload = root.getJSONObject("payload");
+                            if (payload == null) {
+                                return;
+                            }
+                            JSONObject output = payload.getJSONObject("output");
+                            JSONObject sentence = output != null ? output.getJSONObject("sentence") : null;
+                            JSONArray words = sentence != null ? sentence.getJSONArray("words") : null;
+                            JSONObject usage = payload.getJSONObject("usage");
+                            if ((words != null && !words.isEmpty()) || (usage != null && !usage.isEmpty())) {
+                                Log.i(TAG, "STREAM_INPUT_TTS_EVENT_SENTENCE_SYNTHESIS:" + all_response);
+                                high_light_show.buildListFromJson(sentence, "words");
+                                show_text_info.setIndex(sentence, "words"); /* 刷新收到的字幕，UI在playInfo中刷新 */
+                            }
+                        } else {
+                            JSONObject sentence = JSON.parseObject(timestamp);
+                            if (sentence == null) {
+                                return;
+                            }
+                            JSONArray words = sentence != null ? sentence.getJSONArray("words") : null;
+                            if (words != null && !words.isEmpty()) {
+                                Log.i(TAG, "STREAM_INPUT_TTS_EVENT_SENTENCE_SYNTHESIS:" + timestamp);
+                                high_light_show.buildListFromJson(sentence, "words");
+                                show_text_info.setIndex(sentence, "words"); /* 刷新收到的字幕，UI在playInfo中刷新 */
+                            }
+                        }
+                    }
                 } else if (event == StreamInputTtsEvent.STREAM_INPUT_TTS_EVENT_SYNTHESIS_COMPLETE || event == StreamInputTtsEvent.STREAM_INPUT_TTS_EVENT_TASK_FAILED) {
                     is_started = false;
                     /*
@@ -760,11 +933,13 @@ public class DashCosyVoiceStreamTtsActivity extends Activity implements View.OnC
                     if (firstDataFlag) {
                         firstDataFlag = false;
                         firstDataTimestamp = System.currentTimeMillis();
+                        writeDataBytes = 0;
                         Log.i(TAG, "Get first audio data.");
                     }
 
                     mAudioTrack.setAudioData(data);
-//                    Log.d(TAG, "write:" + data.length);
+//                    writeDataBytes += data.length;
+//                    Log.d(TAG, "write:" + data.length + " written:" + writeDataBytes);
                     try {
                         if (mSynthesisAudioFile != null) {
                             mSynthesisAudioFile.write(data);
@@ -779,7 +954,8 @@ public class DashCosyVoiceStreamTtsActivity extends Activity implements View.OnC
             public void onStreamInputTtsLogTrackCallback(Constants.LogLevel level, String log) {
 //                Log.i(TAG, "onStreamInputTtsLogTrackCallback log level:" + level + ", message -> " + log);
             }
-        }, genTicket(), genParameters(), "", Constants.LogLevel.toInt(Constants.LogLevel.LOG_LEVEL_DEBUG), true);
+        }, genTicket(), genParameters(false), "",
+                Constants.LogLevel.toInt(Constants.LogLevel.LOG_LEVEL_VERBOSE), true);
 
         if (Constants.NuiResultCode.SUCCESS != ret) {
             Log.i(TAG, "start tts failed " + ret);
@@ -803,7 +979,7 @@ public class DashCosyVoiceStreamTtsActivity extends Activity implements View.OnC
             public void onStreamInputTtsEventCallback(INativeStreamInputTtsCallback.StreamInputTtsEvent event,
                                                       String task_id, String session_id, int ret_code,
                                                       String error_msg, String timestamp, String all_response) {
-                Log.i(TAG, "stream input tts event(" + event + ") session id(" + session_id + ") task id(" + task_id + ") retCode(" + ret_code + ") errMsg(" + error_msg + ")");
+                Log.d(TAG, "stream input tts event(" + event + ") session id(" + session_id + ") task id(" + task_id + ") retCode(" + ret_code + ") errMsg(" + error_msg + ")");
                 if (event == StreamInputTtsEvent.STREAM_INPUT_TTS_EVENT_SYNTHESIS_STARTED) {
                     Log.i(TAG, "STREAM_INPUT_TTS_EVENT_SYNTHESIS_STARTED");
                     startedTimestamp = System.currentTimeMillis();
@@ -817,7 +993,11 @@ public class DashCosyVoiceStreamTtsActivity extends Activity implements View.OnC
                             }
                             if (mSynthesisAudioFile == null && !curTaskId.isEmpty()) {
                                 try {
-                                    mSynthesisAudioFilePath = debug_path + "/" + "tts_task_id_" + curTaskId + "." + mEncodeType;
+                                    String format = mFormatSpin.getSelectedItem().toString();
+                                    if (format.equals("opus") || format.equals("mp3")) {
+                                        format = "pcm";
+                                    }
+                                    mSynthesisAudioFilePath = debug_path + "/" + "tts_task_id_" + curTaskId + "." + format;
                                     Log.i(TAG, "save tts data into " + mSynthesisAudioFilePath);
                                     mSynthesisAudioFile = new FileOutputStream(mSynthesisAudioFilePath, true);
                                 } catch (IOException e) {
@@ -830,8 +1010,42 @@ public class DashCosyVoiceStreamTtsActivity extends Activity implements View.OnC
                     }
                     mAudioTrack.play();
                     Log.i(TAG, "start play");
+                    show_text_info.clear();
+                    high_light_show.clear();
                 } else if (event == StreamInputTtsEvent.STREAM_INPUT_TTS_EVENT_SENTENCE_SYNTHESIS) {
-                    Log.i(TAG, "STREAM_INPUT_TTS_EVENT_SENTENCE_SYNTHESIS:" + timestamp);
+                    /* 字幕信息可以从all_response中解析，也可从timestamp中解析 */
+                    if (mSubtitlesSwitch.isChecked()) {
+                        if (false) {
+                            JSONObject root = JSON.parseObject(all_response);
+                            if (root == null) {
+                                return;
+                            }
+                            JSONObject payload = root.getJSONObject("payload");
+                            if (payload == null) {
+                                return;
+                            }
+                            JSONObject output = payload.getJSONObject("output");
+                            JSONObject sentence = output != null ? output.getJSONObject("sentence") : null;
+                            JSONArray words = sentence != null ? sentence.getJSONArray("words") : null;
+                            JSONObject usage = payload.getJSONObject("usage");
+                            if ((words != null && !words.isEmpty()) || (usage != null && !usage.isEmpty())) {
+                                Log.i(TAG, "STREAM_INPUT_TTS_EVENT_SENTENCE_SYNTHESIS:" + all_response);
+                                high_light_show.buildListFromJson(sentence, "words");
+                                show_text_info.setIndex(sentence, "words"); /* 刷新收到的字幕，UI在playInfo中刷新 */
+                            }
+                        } else {
+                            JSONObject sentence = JSON.parseObject(timestamp);
+                            if (sentence == null) {
+                                return;
+                            }
+                            JSONArray words = sentence != null ? sentence.getJSONArray("words") : null;
+                            if (words != null && !words.isEmpty()) {
+                                Log.i(TAG, "STREAM_INPUT_TTS_EVENT_SENTENCE_SYNTHESIS:" + timestamp);
+                                high_light_show.buildListFromJson(sentence, "words");
+                                show_text_info.setIndex(sentence, "words"); /* 刷新收到的字幕，UI在playInfo中刷新 */
+                            }
+                        }
+                    }
                 } else if (event == StreamInputTtsEvent.STREAM_INPUT_TTS_EVENT_SYNTHESIS_COMPLETE || event == StreamInputTtsEvent.STREAM_INPUT_TTS_EVENT_TASK_FAILED) {
                     is_started = false;
                     /*
@@ -893,11 +1107,13 @@ public class DashCosyVoiceStreamTtsActivity extends Activity implements View.OnC
                     if (firstDataFlag) {
                         firstDataFlag = false;
                         firstDataTimestamp = System.currentTimeMillis();
+                        writeDataBytes = 0;
                         Log.i(TAG, "Get first audio data.");
                     }
 
                     mAudioTrack.setAudioData(data);
-//                    Log.d(TAG, "write:" + data.length);
+//                    writeDataBytes += data.length;
+//                    Log.d(TAG, "write:" + data.length + " written:" + writeDataBytes);
                     try {
                         if (mSynthesisAudioFile != null) {
                             mSynthesisAudioFile.write(data);
@@ -912,7 +1128,8 @@ public class DashCosyVoiceStreamTtsActivity extends Activity implements View.OnC
             public void onStreamInputTtsLogTrackCallback(Constants.LogLevel level, String log) {
 //                Log.i(TAG, "onStreamInputTtsLogTrackCallback log level:" + level + ", message -> " + log);
             }
-        }, genTicket(), genParameters(), text, "", Constants.LogLevel.toInt(Constants.LogLevel.LOG_LEVEL_DEBUG), true);
+        }, genTicket(), genParameters(true), text, "",
+                Constants.LogLevel.toInt(Constants.LogLevel.LOG_LEVEL_VERBOSE), true);
 
         if (Constants.NuiResultCode.SUCCESS != ret) {
             Log.i(TAG, "play tts failed " + ret);
@@ -938,7 +1155,7 @@ public class DashCosyVoiceStreamTtsActivity extends Activity implements View.OnC
         } else if (view == mVolumeSpin) {
         } else if (view == mFormatSpin) {
             mEncodeType = mFormatSpin.getSelectedItem().toString();
-            stream_input_tts_instance.stopStreamInputTts();
+            stream_input_tts_instance.cancelStreamInputTts();
             mAudioTrack.stop();
             if (!mModelSpin.getSelectedItem().toString().contains("cosyvoice-v1") &&
                     mEncodeType.equals("opus")) {
