@@ -306,7 +306,9 @@ public class SpeechRecognizerActivity extends Activity implements INativeNuiCall
             //nls_config.put("max_start_silence", 10000);
             //nls_config.put("max_end_silence", 800);
 
-            /*若文档中不包含某些参数，但是此功能支持这个参数，可以用如下万能接口设置参数*/
+            // 设置文档中不存在的参数, key为custom_params, value以json string的形式设置参数
+            // 如下示例传入{vocabulary:{"热词1":2,"热词2":2}} 表示在payload下添加参数
+            // payload.vocabulary : {"热词1":2,"热词2":2}
 //            JSONObject extend_config = new JSONObject();
 //            JSONObject vocab = new JSONObject();
 //            vocab.put("热词1", 2);
@@ -483,7 +485,7 @@ public class SpeechRecognizerActivity extends Activity implements INativeNuiCall
             object.put("debug_path", debugpath);
 
             //过滤SDK内部日志通过回调送回到用户层
-            object.put("log_track_level", String.valueOf(Constants.LogLevel.toInt(Constants.LogLevel.LOG_LEVEL_INFO)));
+            object.put("log_track_level", String.valueOf(Constants.LogLevel.toInt(Constants.LogLevel.LOG_LEVEL_NONE)));
 
             // FullMix = 0   // 选用此模式开启本地功能并需要进行鉴权注册
             // FullCloud = 1
